@@ -5,8 +5,9 @@
  * Note to U.S. Government Users Restricted Rights:
  * Use, duplication or disclosure restricted by GSA ADP Schedule
  * Contract with IBM Corp.
- * Copyright (c) 2020 Red Hat, Inc.
  ****************************************************************************** */
+// Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 import _ from 'lodash';
 import crypto from 'crypto';
@@ -19,7 +20,10 @@ const authApiVersion = 'authorization.k8s.io/v1';
 const selfSubjectAccessReviewLink = '/apis/authorization.k8s.io/v1/selfsubjectaccessreviews';
 
 function getGroupFromApiVersion(apiVersion) {
-  return apiVersion.indexOf('/') >= 0 ? apiVersion.split('/')[0] : '';
+  if (apiVersion.indexOf('/') >= 0) {
+    return { apiGroup: apiVersion.split('/')[0], version: apiVersion.split('/')[1] };
+  }
+  return { apiGroup: '', version: apiVersion };
 }
 
 function getApiGroupFromApiVersionOrPath(apiVersion, path, kind) {
@@ -28,6 +32,7 @@ function getApiGroupFromApiVersionOrPath(apiVersion, path, kind) {
   }
   // TODO - need to pass apigroup from backend to this function so we dont need this hack
   let apiGroup = ''; // api group to differentiate between duplicate resources (ie. endpoints & subscriptions)
+  let version = '';
   const pathData = path.split('/');
   // eslint-disable-next-line
   // When splitting the path, the item at pathData[3] is either the api version (if the resource has an apiGroup namespaced or not),
@@ -36,8 +41,13 @@ function getApiGroupFromApiVersionOrPath(apiVersion, path, kind) {
   if (pathData[3] !== kind && pathData[3] !== 'namespaces') {
     // eslint-disable-next-line prefer-destructuring
     apiGroup = pathData[2];
+    // eslint-disable-next-line prefer-destructuring
+    version = pathData[3];
+  } else {
+    // eslint-disable-next-line prefer-destructuring
+    version = pathData[1];
   }
-  return apiGroup;
+  return { apiGroup, version };
 }
 
 export default class GenericModel extends KubeModel {
@@ -223,6 +233,13 @@ export default class GenericModel extends KubeModel {
       });
     }
 
+    if (cluster !== 'local-cluster' && (kind.toLowerCase() === 'secret' || kind.toLowerCase() === 'secrets')) {
+      // We do not allow users to view secrets as this could allow lesser permissioned users to get around RBAC.
+      return [{
+        message: 'Viewing managed cluster secrets is not allowed for security reasons. To view this secret, you must access it from the specific managed cluster.',
+      }];
+    }
+
     // Check if the ManagedClusterView already exists if not create it
     const managedClusterViewName = crypto.createHash('sha1').update(`${cluster}-${name}-${kind}`).digest('hex').substr(0, 63);
 
@@ -233,10 +250,11 @@ export default class GenericModel extends KubeModel {
       throw err;
     });
     if (resourceResponse.status === 'Failure' || resourceResponse.code >= 400) {
-      const apiGroup = getApiGroupFromApiVersionOrPath(apiVersion, path, kind);
+      const { apiGroup, version } = getApiGroupFromApiVersionOrPath(apiVersion, path, kind);
       const response = await this.kubeConnector.managedClusterViewQuery(
         cluster,
         apiGroup,
+        version,
         kind,
         name,
         namespace,
@@ -286,7 +304,7 @@ export default class GenericModel extends KubeModel {
       }
       return localResponse;
     }
-    const apiGroup = getApiGroupFromApiVersionOrPath(apiVersion, path, kind);
+    const { apiGroup, version } = getApiGroupFromApiVersionOrPath(apiVersion, path, kind);
     // Else If updating resource on remote cluster use an Action Type Work
     // Limit workName to 63 characters
     let workName = `update-resource-${this.kubeConnector.uid()}`;
@@ -304,12 +322,12 @@ export default class GenericModel extends KubeModel {
         },
         type: 'Action',
         scope: {
-          resourceType: `${kind}${apiGroup ? `.${apiGroup}` : ''}`,
+          resourceType: apiGroup ? `${kind.toLowerCase()}.${version}.${apiGroup}` : `${kind.toLowerCase()}`,
           namespace,
         },
         actionType: 'Update',
         kube: {
-          resource: `${kind}${apiGroup ? `.${apiGroup}` : ''}`,
+          resource: apiGroup ? `${kind.toLowerCase()}.${version}.${apiGroup}` : `${kind.toLowerCase()}`,
           name,
           namespace,
           template: body,
@@ -338,7 +356,7 @@ export default class GenericModel extends KubeModel {
         const message = _.get(result, 'status.conditions[0].message');
         throw new Error(`Failed to Update ${name}. ${reason}. ${message}.`);
       } else {
-        return _.get(result, 'metadata.name');
+        return _.get(result, 'status.result');
       }
     } catch (e) {
       logger.error('Resource Action Error:', e.message);
@@ -368,7 +386,7 @@ export default class GenericModel extends KubeModel {
       return localResponse;
     }
 
-    const apiGroup = getApiGroupFromApiVersionOrPath(apiVersion, path, kind);
+    const { apiGroup, version } = getApiGroupFromApiVersionOrPath(apiVersion, path, kind);
 
     // Else if deleting resource on remote cluster use Action Type Work
     // Limit workName to 63 characters
@@ -387,12 +405,12 @@ export default class GenericModel extends KubeModel {
         },
         type: 'Action',
         scope: {
-          resourceType: `${kind}${apiGroup ? `.${apiGroup}` : ''}`,
+          resourceType: apiGroup ? `${kind.toLowerCase()}.${version}.${apiGroup}` : `${kind.toLowerCase()}`,
           namespace,
         },
         actionType: 'Delete',
         kube: {
-          resource: `${kind}${apiGroup ? `.${apiGroup}` : ''}`,
+          resource: apiGroup ? `${kind.toLowerCase()}.${version}.${apiGroup}` : `${kind.toLowerCase()}`,
           name,
           namespace,
         },
